#region Copyright
/*
 * Copyright 2005-2010 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
#endregion

using System;
using System.Collections.Generic;
using Seasar.Extension.ADO;
using Seasar.Framework.Aop;
using Seasar.Framework.Log;
using Seasar.Quill.Database.DataSource.Impl;
using Seasar.Quill.Database.Tx;
using Seasar.Quill.Exception;
using Seasar.Quill.Util;

namespace Seasar.Quill
{
    /// <summary>
    /// R|[lgi[ReiNX
    /// </summary>
    /// <remarks>
    /// <para>
    /// i[R|[lg̃CX^X1x
    /// ̂gp(singleton)</para>
    /// </remarks>
    public class QuillContainer : IDisposable
    {
        /// <summary>
        /// O
        /// </summary>
        private static readonly Logger _log = Logger.GetLogger(typeof(QuillContainer));

        // 쐬ς݂ɃR|[lgi[
        protected IDictionary<Type, QuillComponent> components =
            new Dictionary<Type, QuillComponent>();

        // Aspect\zBuilder
        protected AspectBuilder aspectBuilder;

        /// <summary>
        /// QuillContaineȑsRXgN^
        /// </summary>
        public QuillContainer()
        {
            // QuillContainerŎgpAspectBuilder쐬
            aspectBuilder = new AspectBuilder(this);

            //  Quillݒ̏
            QuillConfig.InitializeQuillConfig(this);
            QuillConfig config = QuillConfig.GetInstance();
            LogUtil.Output(_log, "IQLL0003", config.HasQuillConfig());
            if (config.HasQuillConfig())
            {
                //  ݒ񂪂ꍇ̓AZuAf[^\[Xo^
                config.RegisterAssembly();
                RegistDataSource(
                    config.CreateDataSources(),
                    config.GetTransationSettingType());
            }
        }

        /// <summary>
        /// QuillR|[lg擾
        /// </summary>
        /// <remarks>
        /// <para>CX^X̎󂯑TypeƃCX^XTypeꍇ
        /// QuillComponent擾B</para>
        /// <para>QuillR|[lgς݂̏ꍇ͐ς݂
        /// QuillR|[lgԂB</para>        
        /// </remarks>
        /// <param name="type">CX^X̎󂯑Type</param>
        /// <returns>QuillR|[lg</returns>
        public virtual QuillComponent GetComponent(Type type)
        {
            // QuillR|[lg擾ĕԂ
            return GetComponent(type, type);
        }

        /// <summary>
        /// QuillR|[lg擾
        /// </summary>
        /// <remarks>
        /// QuillR|[lgς݂̏ꍇ͐ς݂
        /// QuillR|[lgԂB
        /// </remarks>
        /// <param name="type">CX^X̎󂯑Type</param>
        /// <param name="implType">CX^XType</param>
        /// <returns>QuillR|[lg</returns>
        public virtual QuillComponent GetComponent(Type type, Type implType)
        {
            if (components == null)
            {
                // DestroyĂꍇ͗O𔭐
                throw new QuillApplicationException("EQLL0018");
            }

            lock (components)
            {
                // ɍ쐬ς݂̃CX^Xł邩mF
                if (components.ContainsKey(type))
                {
                    // ɍ쐬ς݂ł΍쐬ς݂̃CX^XԂ
                    return components[type];
                }

                // Aspect쐬iAspectw肳ĂȂ΃TCY0ƂȂ)
                IAspect[] aspects = aspectBuilder.CreateAspects(implType);

                if (implType.IsInterface && aspects.Length == 0)
                {
                    // InterfaceAspect`ĂȂꍇ͗OX[
                    throw new QuillApplicationException("EQLL0008",
                        new string[] { implType.FullName });
                }

                // QuillR|[lg쐬
                QuillComponent component = new QuillComponent(implType, type, aspects);

                // 쐬ς݂QuillR|[lgۑ
                components[type] = component;

                // QuillR|[lgԂ
                return component;
            }
        }

        /// <summary>
        /// QuillContainerQƂj
        /// </summary>
        public virtual void Destroy()
        {
            if (components == null)
            {
                return;
            }

            // ێĂQuillComponent𔽕ׂ̗񋓎q擾
            IEnumerator<QuillComponent> componentValues =
                components.Values.GetEnumerator();

            while (componentValues.MoveNext())
            {
                // QuillComponentDestroyĂяo
                componentValues.Current.Destroy();
            }

            components = null;
            aspectBuilder = null;
        }

        /// <summary>
        /// f[^\[Xo^
        /// </summary>
        public void RegistDataSource(IDictionary<string, IDataSource> dataSources,
            Type defaultTxSettingType)
        {
            // f[^\[X̒`ȂΈȌ͍̏sȂ
            if (dataSources.Count == 0)
            {
                return;
            }
            //  Quillpf[^\[X̐
            SelectableDataSourceProxyWithDictionary dataSourceProxy =
                (SelectableDataSourceProxyWithDictionary)ComponentUtil.GetComponent(
                this, typeof(SelectableDataSourceProxyWithDictionary));
            //  f[^\[X̒`Γo^
            foreach (KeyValuePair<string, IDataSource> dataSourcePair in dataSources)
            {
                dataSourceProxy.RegistDataSource(dataSourcePair.Key, dataSourcePair.Value);
            }

            ITransactionSetting defaultTxSetting = (ITransactionSetting)ComponentUtil.GetComponent(
                this, defaultTxSettingType);
            //  gUNṼftHgݒs
            if (defaultTxSetting != null && defaultTxSetting.IsNeedSetup())
            {
                defaultTxSetting.Setup(dataSourceProxy);
            }
        }
        
        #region IDisposable o

        /// <summary>
        /// ێĂQuillComponentDisposeĂяo
        /// </summary>
        public virtual void Dispose()
        {
            if (components == null)
            {
                // DestroyĂꍇ͗O𔭐
                throw new QuillApplicationException("EQLL0018");
            }

            // ێĂQuillComponent𔽕ׂ̗񋓎q擾
            IEnumerator<QuillComponent> componentValues = 
                components.Values.GetEnumerator();

            while (componentValues.MoveNext())
            {
                // QuillComponentDisposeĂяo
                componentValues.Current.Dispose();
            }
        }

        #endregion
    }
}
